/*********************************************************************
*Project:MG82F6B08_001_104-DEMO
*			MG82F6B08_001_104 SOP16_V11 EV Board (TH221A)
*			CpuCLK=8MHz, SysCLK=8MHz
*Description:
*			EEPROM Test
*Note:
*
*Creat time:
*Modify:
*    
*********************************************************************/
#define _MAIN_C

#include <Intrins.h>
#include <Absacc.h>

#include <Stdio.h>  // for printf

#include ".\include\REG_MG82F6B08_001_104.H"
#include ".\include\Type.h"
#include ".\include\API_Macro_MG82F6B08_001_104.H"
#include ".\include\API_Uart_BRGRL_MG82F6B08_001_104.H"


/*************************************************
*Set SysClk (MAX.24MHz)
*Selection: 
*	8000000,16000000
*	11059200,22118400
*************************************************/
#define MCU_SYSCLK		8000000
/*************************************************/
/*************************************************
*Set CpuClk (MAX.16MHz)
*	1) CpuCLK=SysCLK
*	2) CpuClk=SysClk/2
*************************************************/
#define MCU_CPUCLK		(MCU_SYSCLK)
//#define MCU_CPUCLK		(MCU_SYSCLK/2)

#define TIMER_1T_1ms_TH	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(1000000)))) /256) 			
#define TIMER_1T_1ms_TL	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(1000000)))) %256)

#define TIMER_12T_1ms_TH	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(12000000)))) /256) 			
#define TIMER_12T_1ms_TL	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(12000000)))) %256)

#define LED_R		P32
#define LED_G_0		P33
#define LED_G_1		P46

#define IO_KEY_SAVE			P10

#define EEPROM_TEST_ADDR	0x0000


u8 LedTime;

/***********************************************************************************
*Function:   void INT_T0(void)
*Description:T0 Interrupt handler
*Input:   
*Output:     
*************************************************************************************/
void INT_T0(void) interrupt INT_VECTOR_T0
{
	TH0=TIMER_12T_1ms_TH;
	TL0=TIMER_12T_1ms_TL;
	LED_G_1=!LED_G_1;
	if(LedTime!=0) LedTime--;
}


/*************************************************
*Function: char putchar (char c)   
*Description: send for printf
*Input:     char c
*Output:     
*************************************************/
char putchar (char c)   
{      
	bit bES;
	bES=ES0;
    ES0=0;        
    S0BUF = c;        
    while(TI0==0);        
    TI0=0;        
    ES0=bES;        
    return 0;
}

/*************************************************
*Function:     void DelayXus(u8 xUs)
*Description:  delay,unit:us
*Input:     u8 Us -> *1us  (1~255)
*Output:     
*************************************************/
void DelayXus(u8 xUs)
{
	while(xUs!=0)
	{
#if (MCU_CPUCLK>=11059200)
		_nop_();
#endif
#if (MCU_CPUCLK>=14745600)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=16000000)
		_nop_();
#endif

#if (MCU_CPUCLK>=22118400)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=24000000)
		_nop_();
		_nop_();
#endif		
#if (MCU_CPUCLK>=29491200)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=32000000)
		_nop_();
		_nop_();
#endif

		xUs--;
	}
}

/*************************************************
*Function:     void DelayXms(u16 xMs)
*Description:delay,unit:ms
*Input:     u16 xMs -> *1ms  (1~65535)
*Output:     
*************************************************/
void DelayXms(u16 xMs)
{
	while(xMs!=0)
	{
		CLRWDT();
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		xMs--;
		
	}
}

/***********************************************************************************
*Function:u8 EEPROM_ReadByte(u16 ByteAddr)
*Description:Read EEPROM
*Input:u16 ByteAddr: EEPROM address  (Must be smaller than 0x200)   
*Output:     
*************************************************************************************/
u8 EEPROM_ReadByte(u16 ByteAddr)
{
		bit bEA=EA;
		while((ISPCR&PBSY)!=0);
		IFADRH = ByteAddr>>8;		//Set address high byte
		IFADRL= ByteAddr;			//Set address low byte
		EA = 0;						//Disable interrupt
		IFMT = ISP_READ_EEPROM;
		ISPCR = 0x80; 				//Enable ISP/IAP
		SCMD = 0x46;
		SCMD = 0xB9;
		_nop_();
		EA = bEA; 					//restore interrupt
		while((ISPCR&PBSY)!=0);
		IFMT=0;
		ISPCR = 0;					//Clear ISPCR
		return IFD;
}

/***********************************************************************************
*Function:void EEPROM_WriteByte(u16 ByteAddr,u8 ByteData)
*Description:Write EEPROM
*Input:u16 ByteAddr: Address, u8 ByteData: Data
*Output:     
*************************************************************************************/

void EEPROM_WriteByte(u16 ByteAddr,u8 ByteData)
{
		bit bEA=EA;
		while((ISPCR&PBSY)!=0);
		IFD = ByteData;				//Set data to be wite	
		IFADRH = ByteAddr>>8;		//Set address high byte
		IFADRL= ByteAddr;			//Set address low byte;
		EA = 0;						//Disable interrupt
		IFMT = ISP_WRITE_EEPROM;
		ISPCR = 0x80;               //Enable ISP/IAP
		SCMD = 0x46;
		SCMD = 0xB9;
		_nop_();
		EA = bEA; 					//restore interrupt
		while((ISPCR&EEPF)==0);
		nop();
		IFMT=0;
		ISPCR = 0;					//Clear ISPCR
}

/***********************************************************************************
*Function:   void InitUart0_S0BRG(void)
*Description:   Initialize Uart0, The source of baud rate was S0BRG
*Input:   
*Output:     
*************************************************************************************/
void InitUart0_S0BRG(void)
{
	UART0_SetAccess_S0CR1();			// Set SFR 0xB9 address to access S0CR1
	UART0_SetMode8bitUARTVar();			// 8-bit, Variable B.R.
	UART0_EnReception();				// Enable reception
	UART0_SetBRGFromS0BRG();			// B.R. source: S0BRG
	UART0_SetS0BRGBaudRateX2();			// S0BRG x2
	UART0_SetS0BRGSelSYSCLK();			// S0BRG clock source:SYSCLK
	UART0_SetRxTxP30P31();
	// Sets B.R. value
	UART0_SetS0BRGValue(S0BRG_BRGRL_9600_2X_8000000_1T);

	UART0_EnS0BRG();					// Enable S0BRG
}

/***********************************************************************************
*Function:   void InitPort(void)
*Description:   Initialize IO Port
*Input:   
*Output:     
*************************************************************************************/
void InitPort(void)
{
	PORT_SetP3QuasiBi(BIT0|BIT1|BIT2|BIT3);		// Set P30,P31,P32,P33 as Quasi-Bidirectional,for RXD/TXD,LED
	PORT_SetP4OpenDrainPu(BIT6);				// Set P46 as Open-Drain with pull-up resistor,for LED
	
	PORT_SetP1OpenDrainPu(BIT0);				// Set P10 as Open-Drain with pull-up resistor
	P10=1;
}

/***********************************************************************************
*Function:   void InitTimer0(void)
*Description:Initialize Timer0
*Input:   
*Output:     
*************************************************************************************/
void InitTimer0(void)
{
	TM_SetT0Mode_1_16BIT_TIMER();			// TIMER0 Mode: 16-bit
	TM_SetT0Clock_SYSCLKDiv12();			// TIMER0 Clock source: SYSCLK/12
	TM_SetT0Gate_Disable();

	TM_SetT0LowByte(TIMER_12T_1ms_TL);						// Set TL0 value
	TM_SetT0HighByte(TIMER_12T_1ms_TH);					// Set TH0 value

	TM_EnableT0();							// Enable T0
}


/***********************************************************************************
*Function:   void InitClock(void)
*Description:   Initialize clock	
*Input:   
*Output:     
*************************************************************************************/
void InitClock(void)
{
#if (MCU_SYSCLK==8000000)
	// Default: IHRCO=16MHz, OSCin=IHRCO,MCK=OSCin,MCKDO=MCK/2
#endif

#if (MCU_SYSCLK==11059200)
	// IHRCO=22.12MHz, OSCin=IHRCO,MCK=OSCin,MCKDO=MCK/2
	CLK_SelIHRCO_22118400Hz();
#endif
	
#if (MCU_SYSCLK==16000000)
	// IHRCO=16MHz, OSCin=IHRCO,MCK=OSCin,MCKDO=MCK
	CLK_SetCKCON3(WDTCS_ILRCO|FWKP_NORMAL_120uS|MCKDO_MCK_DIV_1);
#endif

#if (MCU_SYSCLK==22118400)
	// IHRCO=22.12MHz, OSCin=IHRCO,MCK=OSCin,MCKDO=MCK
	CLK_SelIHRCO_22118400Hz();
	CLK_SetCKCON3(WDTCS_ILRCO|FWKP_NORMAL_120uS|MCKDO_MCK_DIV_1);
#endif
	

#if (MCU_CPUCLK==MCU_SYSCLK)
	CLK_SetCpuclk_Sysclk();
#else
	CLK_SetCpuclk_SysclkDiv2();
#endif

	// P45 ouput MCK/4
	//CLK_P45OC_MCKDiv4();
}

/***********************************************************************************
*Function:   void InitInterrupt(void)
*Description: Initialize Interrupt
*Input:   
*Output:     
*************************************************************************************/
void InitInterrupt(void)
{
	INT_EnTIMER0();
}

/***********************************************************************************
*Function:   void InitSystem(void)
*Description:   Initialize MCU
*Input:   
*Output:     
*************************************************************************************/
void InitSystem(void)
{
	InitPort();
	InitClock();
	InitUart0_S0BRG();	
	InitTimer0();
	
	InitInterrupt();
}

void main()
{
	u8 i,rand;
	u16 EEPromAddr;
	
    InitSystem();
    
	INT_EnAll();						// Enable global interrupt
    
	LED_G_0=0;LED_G_1=0;LED_R=0;
	DelayXms(1000);
	LED_G_0=1;LED_G_1=1;LED_R=1;
	printf("\nHello!");


	EEPromAddr=EEPROM_TEST_ADDR;
	rand=EEPROM_ReadByte(EEPromAddr);
	printf("\nEEPROM Read 0x%04X:%02bX",EEPromAddr,rand);
	
    while(1)
    {
    	rand++;
    	if(LedTime==0)
    	{
			LED_G_0=!LED_G_0;
			LedTime=200;
		}
		if(IO_KEY_SAVE==0)
		{
			DelayXms(50);
			if(IO_KEY_SAVE==0)
			{
				printf("\nEEPROM Write 0x%04X:%02bX",EEPromAddr,rand);
				EEPROM_WriteByte(EEPromAddr,rand);
				if(rand==EEPROM_ReadByte(EEPromAddr))
				{
					for(i=0;i<10;i++)
					{
						LED_G_0=0;
						DelayXms(30);
						LED_G_0=1;
						DelayXms(30);
					}
				}
				else
				{
					LED_R=0;
					DelayXms(500);
					LED_R=1;
				}
				while(IO_KEY_SAVE==0)
				{
					DelayXms(50);
				}
			}
		}
    }
}


